defmodule DecentralisedBookIndex.Metadata.BookEditionsRegistry do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer,
    authorizers: [Ash.Policy.Authorizer]

  alias DecentralisedBookIndex.Metadata

  postgres do
    table "book_editions_registries"
    repo DecentralisedBookIndex.Repo
  end

  actions do
    defaults [:read, :create, :update, :destroy]

    create :sync_create do
      accept [:id, :dbi_server_id]
    end

    update :sync do
      accept [:id, :dbi_server_id]
    end

    read :by_id do
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end
  end

  policies do
    bypass actor_attribute_equals(:role, :admin) do
      authorize_if always()
    end

    policy action_type(:read) do
      authorize_if always()
    end

    policy action_type(:create) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:update) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:destroy) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end
  end

  attributes do
    uuid_primary_key :id, writable?: true

    timestamps()
  end

  relationships do
    belongs_to :dbi_server, Metadata.DBIServer

    has_many :books, DecentralisedBookIndex.Metadata.Book
  end
end
