defmodule DecentralisedBookIndexWeb.BookLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
      </.header>

      <.simple_form
        for={@form}
        id="book-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <%= if @form.source.type == :create do %>
          <.input field={@form[:title]} type="text" label="Title" />
          <.input field={@form[:cover_image_url]} type="text" label="Cover image url" />
          <.input
            field={@form[:description]}
            type="textarea"
            label="Description"
          />
          <.bids_inputs form={@form} myself={@myself} />
          <.author_roles_inputs form={@form} myself={@myself} />
          <.input field={@form[:format]} type="text" label="Format" />
          <.input
            field={@form[:language]}
            type="text"
            label="Language"
          />
          <.input field={@form[:page_count]} type="number" label="Page count" />
          <.input field={@form[:publisher_id]} type="text" label="Publisher" />
          <.input
            field={@form[:published]}
            type="date"
            label="Published"
          />
        <% end %>
        <%= if @form.source.type == :update do %>
          <.input field={@form[:title]} type="text" label="Title" />
          <.input field={@form[:cover_image_url]} type="text" label="Cover image url" />
          <.input
            field={@form[:description]}
            type="textarea"
            label="Description"
          />
          <.bids_inputs form={@form} myself={@myself} />
          <.author_roles_inputs form={@form} myself={@myself} />
          <.input field={@form[:format]} type="text" label="Format" />
          <.input
            field={@form[:language]}
            type="text"
            label="Language"
          />
          <.input field={@form[:page_count]} type="number" label="Page count" />
          <.input field={@form[:publisher_id]} type="text" label="Publisher" />
          <.input
            field={@form[:published]}
            type="date"
            label="Published"
          />
        <% end %>

        <:actions>
          <.save_button phx-disable-with="Saving...">
            Save
          </.save_button>
          <.cancel_button phx-click="cancel" phx-target={@myself}>
            Cancel
          </.cancel_button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  def bids_inputs(assigns) do
    ~H"""
    <h2 class="font-semibold leading-8 text-zinc-800 dark:text-white">Book ids</h2>

    <table class="w-full">
      <thead class="border-b border-zinc-100">
        <tr>
          <th class="text-left font-medium text-sm pb-1 px-3 w-36 dark:text-white">Type</th>
          <th class="text-left font-medium text-sm pb-1 px-3 dark:text-white">Id</th>
          <th class="text-left font-medium text-sm pb-1 px-3 w-12"></th>
        </tr>
      </thead>
      <tbody phx-hook="bidSort" id="bidSort" phx-target={@myself}>
        <.inputs_for :let={bid_form} field={@form[:bids]}>
          <tr data-id={bid_form.index}>
            <td class="px-3 w-36">
              <label for={bid_form[:type].id} class="hidden">Type</label>
              <.input field={bid_form[:type]} />
            </td>
            <td class="px-3">
              <label for={bid_form[:bid].id} class="hidden">Id</label>
              <.input field={bid_form[:bid]} />
            </td>
            <td class="flex items-center justify-center w-12 pt-2">
              <.button_link
                phx-click="remove-bid"
                phx-value-path={bid_form.name}
                phx-target={@myself}
                kind="error"
                size="xs"
                inverse
              >
                <svg class="w-6 h-6 text-gray-800 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="currentColor" viewBox="0 0 24 24">
                  <path fill-rule="evenodd" d="M8.586 2.586A2 2 0 0 1 10 2h4a2 2 0 0 1 2 2v2h3a1 1 0 1 1 0 2v12a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V8a1 1 0 0 1 0-2h3V4a2 2 0 0 1 .586-1.414ZM10 6h4V4h-4v2Zm1 4a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Zm4 0a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Z" clip-rule="evenodd"/>
                </svg>
              </.button_link>
            </td>
          </tr>
        </.inputs_for>
      </tbody>
    </table>

    <.add_button phx-click="add-bid" phx-target={@myself} class="w-44">
      Add Book Id
    </.add_button>

    """
  end

  @impl true
  def handle_event("add-bid", params, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.add_form(form, :bids)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("remove-bid", %{"path" => path}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.remove_form(form, path)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("reorder-bids", %{"order" => order}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.sort_forms(form, [:bids], order)
      end)

    {:noreply, socket}
  end

  def author_roles_inputs(assigns) do
    ~H"""
    <h2 class="font-semibold leading-8 text-zinc-800 dark:text-white">Authors</h2>

    <table class="w-full">
      <thead class="border-b border-zinc-100">
        <tr>
          <th class="text-left font-medium text-sm pb-1 px-3 w-96 dark:text-white">Author</th>
          <th class="text-left font-medium text-sm pb-1 px-3 dark:text-white">Role</th>
          <th class="text-left font-medium text-sm pb-1 px-3 w-12"></th>
        </tr>
      </thead>
      <tbody phx-hook="authorRoleSort" id="authorRoleSort" phx-target={@myself}>
        <.inputs_for :let={author_roles_form} field={@form[:author_roles]}>
          <tr data-id={author_roles_form.index}>
            <td class="px-3 w-96">
              <label for={author_roles_form[:author_id].id} class="hidden">Type</label>
              <.input field={author_roles_form[:author_id]} />
            </td>
            <td class="px-3">
              <label for={author_roles_form[:role].id} class="hidden">Id</label>
              <.input field={author_roles_form[:role]} />
            </td>
            <td class="flex items-center justify-center w-12 pt-2">
              <.button_link
                phx-click="remove-author-role"
                phx-value-path={author_roles_form.name}
                phx-target={@myself}
                kind="error"
                size="xs"
                inverse
              >
                <svg class="w-6 h-6 text-gray-800 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="currentColor" viewBox="0 0 24 24">
                  <path fill-rule="evenodd" d="M8.586 2.586A2 2 0 0 1 10 2h4a2 2 0 0 1 2 2v2h3a1 1 0 1 1 0 2v12a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V8a1 1 0 0 1 0-2h3V4a2 2 0 0 1 .586-1.414ZM10 6h4V4h-4v2Zm1 4a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Zm4 0a1 1 0 1 0-2 0v8a1 1 0 1 0 2 0v-8Z" clip-rule="evenodd"/>
                </svg>
              </.button_link>
            </td>
          </tr>
        </.inputs_for>
      </tbody>
    </table>

    <.add_button phx-click="add-author-role" phx-target={@myself} class="w-40">
      Add Author
    </.add_button>
    """
  end

  @impl true
  def handle_event("add-author-role", params, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.add_form(form, :author_roles)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("remove-author-role", %{"path" => path}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.remove_form(form, path)
      end)

    {:noreply, socket}
  end

  @impl true
  def handle_event("reorder-author-role", %{"order" => order}, socket) do
    socket =
      update(socket, :form, fn form ->
        AshPhoenix.Form.sort_forms(form, [:author_roles], order)
      end)

    {:noreply, socket}
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"book" => book_params}, socket) do
    {:noreply, assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, book_params))}
  end

  def handle_event("save", %{"book" => book_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form, params: book_params) do
      {:ok, book} ->
        notify_parent({:saved, book})

        socket =
          socket
          |> put_flash(:info, "Book #{socket.assigns.form.source.type}d successfully")
          |> push_redirect(to: patch_url(socket.assigns.action, book.id))

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  def handle_event("cancel", _params, socket) do
    book = socket.assigns.book
    book_id = if book == nil, do: nil, else: book.id

    socket =
      socket
      |> push_redirect(to: patch_url(socket.assigns.action, book_id))

    {:noreply, socket}
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(%{assigns: %{book: book}} = socket) do
    form =
      if book do
        book =
          book
          |> Ash.load!([:bids, :author_roles, :publisher])

        AshPhoenix.Form.for_update(book, :update, as: "book", actor: socket.assigns.current_user)
      else
        AshPhoenix.Form.for_create(DecentralisedBookIndex.Metadata.Book, :create,
          as: "book",
          actor: socket.assigns.current_user
        )
      end

    assign(socket, form: to_form(form))
  end

  defp patch_url(action, book_id) do
    case action do
      :edit -> ~p"/books/#{book_id}"
      :new  -> ~p"/books"
       _    -> ~p"/books"
    end
  end
end
