defmodule DecentralisedBookIndexWeb.BookLive.Show do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata
  alias DecentralisedBookIndex.Accounts.Role

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      {@book.title}
      <:subtitle>{@authors_string}</:subtitle>

      <:actions>
        <%= if @current_user != nil and Role.can_moderate?(@current_user.role) do %>
          <.link patch={~p"/books/#{@book}/edit"}>
            <.edit_button>
              Edit
            </.edit_button>
          </.link>
        <% end %>
      </:actions>
    </.header>

    <div class="flex flex-wrap flex-[2_1_auto] gap-5">
      <%= if @book.cover_image_url != nil do %>
        <img class="w-36 shadow-lg mt-14 mx-auto md:mx-0 object-cover" src={@book.cover_image_url} alt={"#{@book.title} image"}/>
      <% else %>
        <div class="relative w-36 h-36 overflow-hidden mt-14 mx-auto md:mx-0">
          <svg class="w-36 h-36 text-gray-800 dark:text-white" aria-hidden="true" xmlns="http://www.w3.org/2000/svg" width="24" height="24" fill="currentColor" viewBox="0 0 24 24">
              <path fill-rule="evenodd" d="M6 2a2 2 0 0 0-2 2v15a3 3 0 0 0 3 3h12a1 1 0 1 0 0-2h-2v-2h2a1 1 0 0 0 1-1V4a2 2 0 0 0-2-2h-8v16h5v2H7a1 1 0 1 1 0-2h1V2H6Z" clip-rule="evenodd"/>
          </svg>
        </div>
      <% end %>

      <.list>
        <:item title="Id">{@book.id}</:item>

        <:item title="Title">{@book.title}</:item>

        <:item title="Description">{@book.description}</:item>

        <:item title="Published">{@book.published}</:item>

        <:item title="Language">{@book.language}</:item>

        <:item title="Format">{@book.format}</:item>

        <:item title="Page count">{@book.page_count}</:item>

        <:item title="Cover image url">{@book.cover_image_url}</:item>
      </.list>
    </div>


    <.back navigate={~p"/books"}>Back to books</.back>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok, socket}
  end

  @impl true
  def handle_params(%{"id" => id}, _, socket) do
    book = Ash.get!(Metadata.Book, id, actor: socket.assigns.current_user, load: [:author_roles])
    author_roles = Metadata.get_author_roles_by_book_id!(book.id, load: [:author])
    authors_string = Enum.reduce(
      author_roles, "",
      fn author_role, acc ->
        if acc != "" do
          acc <> ", "
        else
          acc
        end
        <>
        if author_role.role == "" do
          "#{author_role.author.name}"
        else
          "#{author_role.author.name} (#{author_role.role})"
        end
      end
    )

    {:noreply,
     socket
     |> assign(:page_title, page_title(socket.assigns.live_action))
     |> assign(:book, book)
     |> assign(:authors_string, authors_string)
    }
  end

  defp page_title(:show), do: "Show Book"
  defp page_title(:edit), do: "Edit Book"
end
