defmodule DecentralisedBookIndex.Sync.DataTransformers.AuthorTransformerTest do
  use ExUnit.Case, async: true

  alias DecentralisedBookIndex.Sync.DataTransformers.AuthorTransformer

  describe "correct transformations" do
    test "a json contains correct author information" do
      json_body = %{
        "data" => %{
          "attributes" => %{
            "description" => "Something",
            "name" => "Author",
            "avatar_url" => "/images/avatar.png",
            "author_alias_registry_id" => "1320a6fe-b311-45ac-bbaa-39cf29c44624"
          },
          "id" => "889a323e-d104-4b5d-b276-dad5a9b1da9d",
          "links" => %{},
          "meta" => %{},
          "relationships" => %{},
          "type" => "author"
        },
        "jsonapi" => %{"version" => "1.0"},
        "links" => %{
          "self" => "http://localhost:4000/api/v1/json/author/889a323e-d104-4b5d-b276-dad5a9b1da9d"
        },
        "meta" => %{}
      }

      assert {:ok, author} = AuthorTransformer.from_json(json_body)
      assert %{
        id: "889a323e-d104-4b5d-b276-dad5a9b1da9d",
        name: "Author",
        description: "Something",
        avatar_url: "/images/avatar.png",
        author_alias_registry_id: "1320a6fe-b311-45ac-bbaa-39cf29c44624"
      } = author
    end

    test "a json doesn't contains author information \"data\" attribute" do
      json_body = %{
        "attributes" => %{
          "description" => "Something",
          "name" => "Author",
          "avatar_url" => "/images/avatar.png",
          "author_alias_registry_id" => "1320a6fe-b311-45ac-bbaa-39cf29c44624"
        },
        "id" => "889a323e-d104-4b5d-b276-dad5a9b1da9d",
        "links" => %{},
        "meta" => %{},
        "relationships" => %{},
        "type" => "author"
      }

      assert {:ok, author} = AuthorTransformer.from_json(json_body)
      assert %{
        id: "889a323e-d104-4b5d-b276-dad5a9b1da9d",
        name: "Author",
        description: "Something",
        avatar_url: "/images/avatar.png",
        author_alias_registry_id: "1320a6fe-b311-45ac-bbaa-39cf29c44624"
      } = author
    end
  end
end
