defmodule DecentralisedBookIndex.Sync.DataTransformers.BidsTransformerTest do
  use ExUnit.Case, async: true

  alias DecentralisedBookIndex.Sync.DataTransformers.BidsTransformer

  describe "correct transformations" do
    test "a json contains correct bids information" do
      json_body =
        %{
          "data" => [
            %{
              "attributes" => %{"bid" => "1234567890", "order" => 0, "type" => "asin"},
              "id" => "0c0c0b3a-0410-4cb0-a7e3-b62bb392b097",
              "links" => %{},
              "meta" => %{},
              "relationships" => %{},
              "type" => "book_id"
            },
            %{
              "attributes" => %{"bid" => "1234567890123", "order" => 1, "type" => "isbn13"},
              "id" => "31eafe52-0f39-49ad-8980-cdd448f9c738",
              "links" => %{},
              "meta" => %{},
              "relationships" => %{},
              "type" => "book_id"
            }
          ],
          "jsonapi" => %{"version" => "1.0"},
          "links" => %{
            "self" =>
              "http://localhost:4000/api/v1/json/books/233987a5-f318-460b-9973-e553c99a0eaf/bids"
          },
          "meta" => %{}
        }

      assert {:ok, bids} = BidsTransformer.from_json(json_body)

      assert [
               %{
                 id: "0c0c0b3a-0410-4cb0-a7e3-b62bb392b097",
                 order: 0,
                 type: "asin",
                 bid: "1234567890"
               },
               %{
                 id: "31eafe52-0f39-49ad-8980-cdd448f9c738",
                 order: 1,
                 type: "isbn13",
                 bid: "1234567890123"
               }
             ] = bids
    end
  end
end
