defmodule DecentralisedBookIndex.Sync.DataTransformers.BookEditionsRegistrySyncTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Sync.BookEditionsRegistrySync
  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints
  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "sync book edition registry transformations" do
    test "a new registry will be created" do
      server = generate(dbi_server(url: @test_server_endpoint))

      attrs = %{
        id: "889a323e-d104-4b5d-b276-dad5a9b1da9d",
      }

      assert :ok = BookEditionsRegistrySync.create_update(attrs, server.id)
      assert {:ok, saved_alias} = Metadata.get_book_editions_registry_by_id(attrs.id)
      assert server.id == saved_alias.dbi_server_id
    end

    test "the same registry", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, same_alias} = Metadata.create_book_editions_registry(actor: user)

      attrs = %{
        id: same_alias.id,
      }

      assert :ok = BookEditionsRegistrySync.create_update(attrs, server.id)
      assert {:ok, saved_alias} = Metadata.get_book_editions_registry_by_id(attrs.id)
      assert server.id == saved_alias.dbi_server_id
    end
  end
end
